#ifndef HOST_H
#define HOST_H

#include <string>
#include "thread.h"
#include "mutex.h"
#include "socket.h"

// Controls the client's interface to the host
extern class Host {
public:
    
    Host();
    ~Host();
    
    // Connects to a server specified by ip on port port.  Returns false if
    // it was unable to connect.  Use net::getError() for a description of the
    // error.
    bool connect(const std::string &ip, unsigned short port = 1423);
    
    // call me!
    // 
    // Returns false if any error has occurred
    //
    // If error is non-zero it will be set to the issue that happened
    bool poll(int *error = 0);
    
    // Tries to login with username and password.  check loginFail and loginSuccess
    // until one of them returns true.
    void login(const std::string &username, const std::string &password);
    
    // True if the event happens
    bool loginFailure();
    bool loginSuccess();
    
    /* Server event system  functions */
    // Attempts to make a new user on the server with username and password.
    void newUser(const std::string &username, const std::string &password);
    
    /// Attempt to purchase and place a building
    void buyBuilding(unsigned int type, int x, int y);
    /// Attempt to purchase and place a train
    void buyTrain(int x, int y);
    /// Attempt to purchase and place a track
    void buyTrack(int x, int y);
    
    // Attempt to remove a building
    void removeBuilding(unsigned int id);
    
    /// Attempt to place a track piece at x / y.
    void placeTrack(int x, int y);
    
    // Attempt to remove a track piece
    void removeTrack(unsigned int id);
    
    // Returns 0 if the last server event system function was successfull, -1
    // if it is still pending or a positive number that can be looked up in
    // errors::list the system event did not complete successfully.
    int check();
    
    // Don't use stuff below
    
    net::Socket con;
    
    Thread *reciever;
    
    int loginStatus;
    int statusResponse;
    
} host;

#endif
