#include "host.h"
#include "clihandler.h"
#include "log.h"
using namespace std;
using namespace net;

Host host;

Host::Host()
    : con(Socket::TCP), loginStatus(0)
{
    
}

Host::~Host()
{
    
}

bool Host::connect(const string &ip, unsigned short port)
{
    loginStatus = 0;
    
    if(!con.connect(ip, port))
        if(!con.connectIp(ip, port))
            return false;
    
    return true;
}

bool Host::poll(int *error)
{
    bool ready;
    
    if(!select(&con, &con + 1, &ready)) {
        
        if(error)
            *error = getErrno();
        
        out << "Select errored\n";
        
        return false;
    }
    else if(ready) {
        
        char msg[256];
        char tag;
        
        if(con.recv(&tag, 1) < 1) {
            
            out << "recv(tag) errored\n";
            
            if(error)
                *error = getErrno();
            
            return false;
        }
        
        if(tag < 1 || tag > maxTag) {
            
            out << "Invalid tag: " << (int)tag << "\n";
            return true;
        }
        
        out << "Tag: " << (int)tag << "\n";
        
        if(con.recv(msg, sizeOfData[(int)tag]) < 1) {
            
            if(error)
                *error = getErrno();
            
            out << "recv(msg) errored\n";
            
            return false;
        }
        
        tagHandlers[(int)tag](msg, Socket(con, false));
    }
    
    return true;
}

int Host::check()
{
    int ret = statusResponse;
    
    statusResponse = 0;
    
    return ret;
}

void Host::login(const string &name, const string &pass)
{
    char msg[65];
    
    *msg = 1; // login
    
    strcpy(msg + 1, name.c_str());
    strcpy(msg + 33, pass.c_str());
    
    for(int i = strlen(msg + 1); i < 32; i++)
        msg[i + 1] = '\0';
    
    for(int i = strlen(msg + 33); i < 32; i++)
        msg[i + 33] = '\0';
    
    if(!con.send(msg, sizeof(msg)))
        loginStatus = -1;
    else
        loginStatus = 0;
}

bool Host::loginFailure()
{
    
    bool ret = loginStatus < 0;
    
    
    return ret;
}

bool Host::loginSuccess()
{
    
    bool ret = loginStatus > 0;
    
    
    return ret;
}

void Host::newUser(const string &name, const string &pass)
{
    char msg[65];
    
    *msg = 2;
    
    strcpy(msg + 1, name.c_str());
    strcpy(msg + 33, pass.c_str());
    
    for(int i = strlen(msg + 1); i < 32; i++)
        msg[i + 1] = '\0';
    
    for(int i = strlen(msg + 33); i < 32; i++)
        msg[i + 33] = '\0';
    
    if(!con.send(msg, sizeof(msg)))
        statusResponse = 22;
    else
        statusResponse = -1;
}

void Host::buyBuilding(unsigned int type, int x, int y)
{
    char msg[13];
    
    *msg = 4;
    
    unsigned int t2 = htonl(type);
    int x2 = htonl(x);
    int y2 = htonl(y);
    
    memcpy(msg + 1, (char*)&t2, 4);
    memcpy(msg + 5, (char*)&x2, 4);
    memcpy(msg + 9, (char*)&y2, 4);
    
    if(!con.send(msg, sizeof(msg)))
        statusResponse = 22;
    else
        statusResponse = 0;
}

void Host::removeBuilding(unsigned int id)
{
    char msg[5];
    
    *msg = 6;
    
    out << "remove: " << id << "\n";
    
    unsigned int i = htonl(id);
    
    memcpy(msg + 1, (char*)&i, 4);
    
    if(!con.send(msg, sizeof(msg)))
        statusResponse = 22;
    else
        statusResponse = 0;
}

void Host::placeTrack(int x, int y)
{
    statusResponse = 0;
}

void Host::buyTrack(int x, int y)
{
    statusResponse = 0;
}

void Host::removeTrack(unsigned int id)
{
    statusResponse = 0;
}
