#ifndef ANIMATION_H
#define ANIMATION_H

#include "datafile.h"
#include "stopwatch.h"
#include "bitmap.h"
#include <vector>
#include <allegro.h>

class animation {
public:
    
    // path must be ready to use with the specified datafile.
    // The path will be postfixed with a number starting at 0 and
    // incremented until no image is found.  An example path could
    // be: CHARACTERS/BRILLY/FIST/LEFT/
    // 
    // Note the object in the datafile must have an attribute called
    // TIME which specifies how many milliseconds the frame will display
    // for.  If it is not specified the time defaults to 0
    animation(const std::string &path, datafile_t* = &datafile);

    animation();
    animation(const animation&);
    
    // Animation is running (started) by default
    void start();
    void restart();
    void unpause();
    void stop();
    void pause();
    
    // Alais functions representing the same operations
    bool unpaused();
    bool paused();
    bool started();
    bool stopped();
    
    // Calculates the current Bitmap and returns it
    Bitmap *get_bmp()
    {
        if(frames.empty())
            return 0;
        
        unsigned int s = frames.size();
        
        while(running && frame_msec_start + frames[cur_frame].msec_delay < *msecs) {
            
            frame_msec_start += frames[cur_frame].msec_delay;
            
            cur_frame++;
            
            if(cur_frame >= s)
                cur_frame = 0;
            
            if(!frames[cur_frame].msec_delay)
                break;
        }
        
        return &frames[cur_frame].bmp;
    }
    
    Bitmap *get_bmp(unsigned int frame)
    {
        if(frames.empty())
            return 0;
        
        return &frames[frame % frames.size()].bmp;
    }
    
    // Must be set to point to a variable that holds the milliseconds since
    // program start
    static volatile unsigned int *msecs;
    
private:
    
    struct Frame {
        
        int msec_delay;
        Bitmap bmp;
        
        Frame(BITMAP *tmp) : bmp(tmp) {}
    };
    
    std::vector<Frame> frames;
    
    unsigned int cur_frame;
    unsigned int frame_msec_start;
    
    bool running;
};

#endif
