#ifndef EMITTER_H
#define EMITTER_H

#include "particle.h"
#include <list>

class EmitManager;

// Controls a particle effect by manipulating a list of particles
class Emitter {
public:
    
    // Loads the emitter with the specified program ID.  The program
    // ID basically tells what kind of particle this emitter should
    // create.
    Emitter(int programID, int x, int y);
    
    // Called at a constant rate in order to update the emitter.  Handler
    // is used to add new particles (see the comments for EmitManager for
    // more details on this).
    void update(EmitManager &handler);
    
    // True when the emitter is finished and this instance can be deleted
    bool done;
    
    // The position to create the effect at
    int x, y;
    
public:
    
    // The 'program' to be run by this emitter.  Basically it means which
    // effect this emitter should create
    int programID;
};

#endif
