#ifndef FONT_H
#define FONT_H

#include "datafile.h"
#include "bitmap.h"
#include <map>

class FontRenderer {
public:
    
    // Loads the font from the specified objname from the specified datafile.
    // Returns the number of letters loaded.  The objname must end in a forward slash
    // (unless the font is in the root datafile) and the datafile under that / must contain
    // ascii values directly.  Upper case letters must be followed by a 'u'.
    int load(const std::string &objname = "fonts/font1/", datafile_t *datafile = &datafile);
    
    // Like load but only starts a load, the loadStep method should be used to finish the load.
    void startLoad(const std::string &objname = "fonts/font1/", datafile_t *datafile = &datafile);
    
    // Non-blocking version of load
    // Returns -1 if should continue loading.  Must be called after startLoad.  Keep calling it
    // untill it returns something else.  When it is finished it will return the number of glyphs
    // loaded.
    int loadStep();
    
    // Unloads the font, freeing all Bitmaps associated with it.  This function is called
    // automaticaly by the destructor.
    void unload();
    
    // Unloads the font through the method unload.
    ~FontRenderer();
    
    // Attempts to draw the specified letter to the screen.  X and Y specify the distance
    // from the upper lefthand corner of the screen in pixels.  tint specifies the
    // color of the letter to be drawn (see bitmap.h for instructions on how to use this).
    void draw(const char letter, int x, int y, const Tint &tint = Tint());
    
    // Like draw but draws a string of letters
    void drawString(const std::string &str, int x, int y, const Tint &tint = Tint());
    
    // Returns the height in pixels of this font.  Returns 0 if the font height is 0 or there
    // is no font loaded.
    int height();
    
    // Returns the width in pixels of this letter in this font.  Returns 0 if the font
    // width is 0 or there is no font loaded.
    int width(const char letter);
    
    // Returns the width in pixels of this string in this font.  Returns 0 if the font
    // width is 0 or none of the letters specified are loaded or there is no font loaded.
    int stringWidth(const std::string &str);

private:
    
    std::map<char, Bitmap*> letters;
    
    char loadStepPosition;
    std::string objname;
    datafile_t *datf;
};

#endif
