#ifndef GUI_H
#define GUI_H

#include "bitmap.h"
#include <stack>

class GuiState;

class Gui {
public:
    
    // Starts running state.  This function blocks untill the gui is done
    // proccessing.
    Gui(GuiState *state);
    
    // For use by GuiStates in order to reset the timer
    void resetTimer();
    
    // Returns the value returned by GuiState::getReturnCode
    int returnCode();
    
private:
    
    bool timerNeedsReset;
    
    GuiState *guiState;
};

class GuiState {
public:
    
    // Return code for think method
    enum Options { CONTINUE, END };
    
    virtual ~GuiState();
    
    // Optional method called when this GuiState needs updating.  Method
    // should draw to the screen.
    virtual void draw() {}
    
    // Called at a constant rate (probably once every 20 milliseconds).
    //
    // Returns CONTINUE to continue running or END to end this state.
    virtual Options think(Gui &caller) = 0;
    
    // Optional method to supply a return code to the calling function.
    virtual int getReturnCode() { return 0; }
    
    // Optional method to recieve the running frames per second.
    virtual void fps(int) {}
};

#endif
