#ifndef PARTICLE_H
#define PARTICLE_H

#include "animation.h"
#include "datafile.h"
#include "stopwatch.h"
#include "timer.h"

#define PARTICLE_CL_FADE_IN   (1.0-(cur/ini))
#define PARTICLE_CL_FADE_OUT  (cur/ini)
#define PARTICLE_CL_BLINK     ((ttl[0]<(ttl[1]/2.0f))?(cur/(ini/2.0f)):((ini/2.0f)-((cur-(ini/2.0))))/(ini/2.0f))

class CirclesBase;

// Class that handles a single particle effect.  Used internaly by Emitter.
// If you're trying to make particle effects look into EmitManager and Emitter.
class Particle {
public:
    
    // Loads the specified object from the specified datafile.  The
    // datafile defaults to the default global datafile.
    void load(const std::string &objname, datafile_t* = &datafile);
    
    // Cleans up graphic's that are loaded into memory.  Must be called to prevent
    // memory leaks.
    static void cleanup();
    
    // Draws the particle to the screen
    void draw(const Tint &screenTint = Tint());
    
    // Draws the particle to the screen but drawing to the position
    // accounting for the rotation amount of the passed Circles instance.
    void drawRotated(const CirclesBase *circles, const Tint &screenTint = Tint());
    
    // Called a constant rate in order to update the particle's logic
    void update();
    
    // Intial position
    short xStart, yStart;
    
    // Current position
    float x, y;
    
    // Current transparency of particle
    float alpha;
    
    // Timer for life of particle.
    unsigned char ttl[2];
    
    // width and height this particle should be stretched to
    float width, height;
    
    // Direction particle is moving
    float dir;
    
    // speed (or velocity) particle is moving at
    float vel;
    
    // The type of particle.  The most significant bit represents whether or not
    // this particle is dead.  The second most significant bit represents whether
    // or not this particle should be drawn with additive blending.
    unsigned char logic;
    
private:
    
    // The graphic for the particle
    animation *graphic;
};

#endif
