#ifndef SETTINGS_H
#define SETTINGS_H

#include <string>

extern class Settings {
public:
    
    // filename is the file where settings can be found.
    std::string filename;
    
    // Defaults all values
    Settings();
    
    // Saves settings to file.
    ~Settings();
    
    // Loads settings from file
    void load();
    
    // Saves settings to file
    void save();
    
    // Applies settings after having modified them
    void apply();
    
    /* Various changable settings */
    
    // True if all sound is muted.
    bool mute;
    
    // 1 is full volume, 0 is effectively same as muting.
    float sfxVolume;
    
    // 1 is full volume, 0 is effectively same as muting.
    float musicVolume;
    
    // Width of screen in pixels
    int screenWidth;
    
    // Height of screen in pixels
    int screenHeight;
    
    // Color depth of screen (8, 15, 16, 24, or 32)
    int colorDepth;
    
    // True if fullscreen, false if windowed
    bool fullscreen;
    
    // True if fps should be displayed
    bool fpsDisplay;
    
    // True if averaging should be done for the fps display (to make it less
    // jumpy)
    bool averageFps;
	
	// True if particle effects are turned on
	bool particleEffects;
	
	// server to connect to
	std::string server;
	
	// port to connect on
	unsigned short port;
	
	// True balls should be drawn twice.
	bool debugTest1;
    
    Settings &operator =(Settings &settings);
    
private:
    
    /* Attributes for telling what settings have been changed.  For use by apply */
    
    int oldScreenWidth, oldScreenHeight;
    int oldColorDepth;
    bool oldFullscreen;
    
} settings;

#endif
