#ifndef TERMINAL_H
#define TERMINAL_H

#include <vector>
#include "user.h"
class Track;


/// Stores ticket price information for destinations at a terminal
struct CityLink {
    /// Player who owns this link
    unsigned int player;
    /// The ID of the city this link represents
    unsigned int cityA;
    /// The ID of the other city this link represents
    unsigned int cityB;
    /// The cost of using this route
    unsigned int cost;
};

class Terminal {
public:
    Terminal();
    Terminal(unsigned int owner_id_, unsigned int city_, unsigned int track_);
    
    ~Terminal();
    
    /// Adds the list of possible city exits to the exits parameter.
    void list_exits(std::vector<CityLink>& exits);
    /// @returns the cost of riding on the given track
    unsigned int get_cost(unsigned int track);
    /// Adjusts the cost of the given track's ticket price
    /// @param updateboth Unimplemented. Don't set this to true.
    void adjust_ticket(unsigned int track, unsigned int newcost, bool updateboth = false);
    /// Recalculates all possible routes reachable from this city
    void recalculate_tracks();
    /// Used to tell other temrinals about links
    void propagate(std::vector<Terminal*>& newtracks, std::vector<unsigned int>& buildings);
    
    /// ID of the user who owns this terminal
    unsigned int owner_id;
    /// The city the terminal is in
    unsigned int city;
    /// Track this terminal leads out to
    unsigned int track;
    /// All the places this terminal can go to, with prices
    std::vector<CityLink> tracks;
    /// All the boring places this terminal can go to
    std::vector<unsigned int> buildings;
};

#endif
