#ifndef WIDGETS_INPUT_H
#define WIDGTES_INPUT_H

#include "widget.h"
#include "animation.h"
#include "log.h"

namespace widgets {
    
    // A Input is a box that can have text inputed into it
    class Input : public Widget {
    public:
        
        Input();
        virtual ~Input() { }
        
        // Attributes can contain the following key(s):
        // "value", string value in the Input box
        
    protected:
        
        // Called when the widget needs to be drawn.  xstart and ystart are added to the
        // widget's x / y position to deduce the drawing position.
        virtual void draw(int xstart, int ystart, const Tint &tint);
        
        // The events this widget needs to recieve
        virtual Action onHover();
        virtual Action onClick();
        virtual Action onIdle();
        
        // Syncs the 'value' attribute if nessicary and moves the cursor to the end of the
        // input box
        virtual void load();
        
    private:
        
        // Syncs the value of this widget with the widget specified in
        // the attribute 'syncValueWith'.  The value of 'syncValueWith'
        // should be the same as the value of 'name' on the widget
        // to sync with.
        void syncValue();
        
        // Returns the number of characters that are visible given the current values of
        // cursorPos and scrollPos
        int visibleChars();
        
        // Gets the width in pixels of a given string, converting the string to '*'s if
        // password mode is set.
        int getWidth(std::string str);
        
        // Same as getWidth(std::string) but takes one character instead of a string
        int getWidth(char c);
        
        // Returns the cursor position in pixels from the left edge of the Input widget.
        int getCursorPixel();
        
        // Wraps setCursorPos but takes a parameter x that represents the distance in
        // pixels from the left edge of the Input widget.
        //
        // See the comments for setCursorPos for a better description of what this
        // method will do.
        void setCursorPixel(int x);
        
        // Sets the CursorPos value to newPos first checking to make sure
        // that newPos is in the correct range (0 being far left, n being far right, where
        // n is the length of the 'value' string).
        //
        // setCursorPos also calls boundsCheckScroll to make sure scrollPos is still in
        // the correct range.
        void setCursorPos(int newPos);
        
        // Changes scrollPos untill is within the correct range.
        void boundsCheckScroll();
        
        int xstart, ystart;
        
        // These are both in character counts and are *not* in pixel counts.
        int cursorPos, scrollPos;
        
        animation textLeft, textRight;
        animation left, center, right;
        animation cursor;
    };
}

#endif
