#ifndef WIDGETS_SLIDER_H
#define WIDGETS_SLIDER_H

#include "widget.h"
#include "animation.h"

namespace widgets {
    
    class Slider : public Widget {
    public:
        
        Slider();
        virtual ~Slider() {}
        
        // Attributes can contain the following keys:
        // "value", Floating point value representing the current value of the slider.
        // "1.0" is full "0.0" is empty
        
    protected:
        
        // Called when the widget needs to be drawn.  xstart and ystart are added to the widget's
        // x / y position to deduce the drawing position.
        virtual void draw(int xstart, int ystart, const Tint &tint);
        
        // The events this widget needs to recieve
        virtual Action onHover();
        virtual Action onLeave();
        virtual Action onClick();
        virtual Action onRelease();
        
    private:
        
        // True if the mouse is over the decrement button
        bool overDecrement();
        
        // True if the mouse is over the increment button
        bool overIncrement();
        
        // True if the mouse is over the center.  where will be set to the position over
        // the center the mouse has.  1 is as far right as is possible and 0 is as far
        // left as possible.
        bool overCenter(float *where);
        
        float f;
        
        int xstart, ystart;
        
        // True when the widget starts being clicked and set to false
        // once the mouse has left the widget
        bool hasClicked;
        
        animation left, right;
        animation center, meter;
        
        animation minibuttonActive;
        animation minibuttonInactive;
        animation minibuttonHover;
        
        animation *leftMinibutton;
        animation *rightMinibutton;
    };
}

#endif
