#include "emitmanager.h"
#include "log.h"
#include "settings.h"
using namespace std;

int EmitManager::particleCount = 0;

EmitManager::~EmitManager()
{
    clearEmitters();
    clearParticles();
}

Emitter &EmitManager::add(int program, int x, int y)
{
	emitters.push_back(new Emitter(program, x, y));
	
	return *emitters.back();
}

void EmitManager::update()
{
    {
        list<Emitter*>::iterator itr = emitters.begin();
        
        while(itr != emitters.end()) {
            
            (*itr)->update(*this);
            
            if((*itr)->done) {
                
                delete *itr;
                itr = emitters.erase(itr);
            }
            else
                ++itr;
        }
    }
    
    {
        list<Particle*>::iterator itr = particles.begin();
        
        while(itr != particles.end()) {
            
            if(!((*itr)->logic & 0x80)) {
                
                (*itr)->update();
                
                if(--(*itr)->ttl[0] == 0) {
                    
                    (*itr)->logic |= 0x80;
                }
            }
            
            ++itr;
        }
    }
}

void EmitManager::draw(const Tint &screenTint)
{
    for(list<Particle*>::reverse_iterator itr = particles.rbegin(); itr != particles.rend(); ++itr)
        if(!((*itr)->logic & 0x80))
            (*itr)->draw(screenTint);
}

Particle *EmitManager::addParticle(const string &objname, datafile_t *datafile)
{
    if(!settings.particleEffects)
        return 0;
    
    list<Particle*>::iterator itr = particles.begin();
    
    while(itr != particles.end()) {
        
        if((*itr)->logic & 0x80) {
            
            (*itr)->load(objname, datafile);
            
            return *itr;
        }
        
        ++itr;
    }
    
    particles.push_back(new Particle);
    particles.back()->load(objname, datafile);
    
    particleCount++;
    
    return particles.back();
}

void EmitManager::clearEmitters()
{
    for(list<Emitter*>::iterator itr = emitters.begin(); itr != emitters.end(); ++itr)
        delete *itr;
    
    emitters.clear();
}

void EmitManager::clearParticles()
{
    particleCount -= particles.size();
    
    for(list<Particle*>::iterator itr = particles.begin(); itr != particles.end(); ++itr)
        delete *itr;
    
    particles.clear();
}
