#include <allegro.h>
#ifdef WIN32
#include <winalleg.h>
#endif
#include "guimanager.h"
#include "guitilemap.h"
#include "gameplay.h"
#include "log.h"
#include "sounds.h"
#include "message.h"
#include "errors.h"
#include "print.h"
#include "settings.h"
#include "sounds.h"
#include "conf.h"
#include "host.h"
#include "settings.h"
#include "socket.h"
#include <sstream>
#include <iostream>
#include <algorithm>
#include <cmath>
using namespace std;

GuiManager::GuiManager()
    :  cursor("gui/cursor/"), shouldClearScreen(0), quit(false)
{
    screenStack.push_back(new GuiScreen("gui/entry"));
}

GuiManager::~GuiManager()
{
    while(!screenStack.empty()) {
        
        delete screenStack.back();
        screenStack.erase(--screenStack.end());
    }
}

void GuiManager::draw()
{
    globalConfInstance->clearScreen();
    
    GuiScreen *guiScreen = 0;
    
    if(!screenStack.empty())
        guiScreen = screenStack.back();
    
    if(settings.fpsDisplay)
        Print(10, 10) << "Fps: " << currentFPS;
    
    if(guiScreen)
        guiScreen->draw();
    
    globalConfInstance->flip();
}

GuiManager::Options GuiManager::think(Gui &caller)
{
    GuiScreen *screen = 0;
    
    if(!screenStack.empty())
        screen = screenStack.back();
    
    if(screen)
        handleActions();
    
    if(key[KEY_ESC] || quit)
        return END;
    
    return CONTINUE;
}

void GuiManager::handleActions()
{
    if(screenStack.empty())
        return;
    
    GuiScreen *screen = screenStack.back();
    
    screen->poll();
    
    if(screen->actionWaiting()) {
        
        Widget *commander;
        
        string action = screen->getAction(&commander);
        
        PerformAction: {
            
            out << "Action \"" << action << "\" activated.\n";
            
            if(commander->attributes["special"] == "disabled") {
                
                out << "Action canceled (the widget was disabled).\n";
            }
            else if(commander->attributes.count("requireAttribute") && commander->attributes[commander->attributes["requireAttribute"]].empty()) {
                
                out << "Action canceled (a requried attribute was missing from widget).\n";
            }
            else if(action == "runLogic") {
                
                Widget *widget = 0;
                
                widget = screen->findByKey("name", commander->attributes["logic"]);
                
                if(widget) {
                    
                    action = widget->attributes["activationAction"];
                    commander = widget;
                    
                    goto PerformAction;
                }
                else
                    out << "Warning, unable to find Logic widget for runLogic action.\n";
            }
            else if(action == "play") {
                
                // TODO
            }
            else if(action == "editor") {
                
                // TODO
            }
            else if(action == "load") {
                
                // TODO
            }
            else if(action == "connect") {
                
                out << "Connecting to " << settings.server << " port "
                    << settings.port << "\n";
                    
                if(!host.connect(settings.server, settings.port))
                {
                    stringstream str;
                    str << "Failed to connect to server " << settings.server
                        << ":" << settings.port << ".\n" << net::getError();
                    
                    out << str.str();
                    
                    message(str.str());
                    quit = true;
                }
                else
                    out << "Success.\n";
            }
            else if(action == "newUser") {
                
                Widget *username, *pass;
                
                username = screen->findByKey("name", commander->attributes["username"]);
                pass = screen->findByKey("name", commander->attributes["pass"]);
                
                if(!username || !pass)
                    err << "Invalid username or pass widget!\n";
                else {
                    
                    out << "Creating user: " << username->attributes["value"] << "\n";
                    
                    host.newUser(username->attributes["value"], pass->attributes["value"]);
                    
                    int i;
                    int error;
                    
                    while((i = host.check()) == -1)
                        if(!host.poll(&error)) {
                            
                            out << "Lost the connection! shit!\n"
                                << net::getError(error) << "\n";
                            i = -1;
                            break;
                        }
                    
                    if(i == 0)
                        message("Username successfully created.  Now login.");
                    else if(i > 0)
                        message(errors::list[i]);
                }
            }
            else if(action == "login") {
                
                Widget *username, *pass;
                
                username = screen->findByKey("name", commander->attributes["username"]);
                pass = screen->findByKey("name", commander->attributes["pass"]);
                
                if(!username || !pass)
                    err << "Invalid username or pass widget!\n";
                else {
                    
                    host.login(username->attributes["value"], pass->attributes["value"]);
                    
                    int error;
                    
                    for(;;) {
                        
                        if(host.loginSuccess()) {
                            
                            GuiTilemap guiTilemap;
                            Gui gui(&guiTilemap);
                            break;
                        }
                        else if(host.loginFailure()) {
                            
                            message("Bad username or password.");
                            break;
                        }
                        
                        if(!host.poll(&error)) {
                            
                            out << "Lost the connection! shit!\n"
                                << net::getError(error) << "\n";
                            break;
                        }
                    }
                }
            }
            else if(action == "previous") {
                
                int prevcount = atoi(commander->attributes["prevcount"].c_str());
                
                if(prevcount < 0) {
                    
                    while(screenStack.size() > (unsigned)abs(prevcount) + 1)
                        screenStack.erase(----screenStack.end());
                }
                else {
                        
                    for(int i = prevcount - 1; i > 0; i--) {
                        
                        if(screenStack.size() <= 1)
                            break;
                        
                        screenStack.erase(----screenStack.end());
                    }
                }
                
                if(screenStack.size() <= 1)
                    ;//TODO (quit)
                
                //TODO
            }
            else if(action == "quit") {
                
                // TODO
            }
            else if(action == "toggleMute") {
                
                if(commander->attributes["checked"] == "0")
                    settings.mute = false;
                else
                    settings.mute = true;
            }
            else if(action == "toggleParticles") {
                
                if(commander->attributes["checked"] == "0")
                    settings.particleEffects = false;
                else
                    settings.particleEffects = true;
            }
            else if(action == "toggleFullscreen") {
                
                if(commander->attributes["checked"] == "0")
                    settings.fullscreen = false;
                else
                    settings.fullscreen = true;
            }
            else if(action == "play") {
            
                GuiTilemap guiTilemap;
                Gui gui(&guiTilemap);
            }
            else {
                
                out << "Warning: \"" << action << "\" could not be found.\n";
            }
        }
    }
}
