#include <allegro.h>
#ifdef WIN32
#include <winalleg.h>
#endif
#include "guitilemap.h"
#include "gameplay.h"
#include "log.h"
#include "sounds.h"
#include "print.h"
#include "settings.h"
#include "sounds.h"
#include "conf.h"
#include "world.h"
#include "software.h"
#include "message.h"
#include "host.h"
#include "errors.h"
#include <sstream>
#include <iostream>
#include <algorithm>
#include <cmath>
using namespace std;

const int brow_h = 32 + 32 * 6;

GuiTilemap::GuiTilemap()
    :  old_mouse_b(0), cursor("gui/cursor/"), handles("gui/handles/"), bg("gui/background/"),
    cursor_state(0), building(1),
    shouldClearScreen(0)
{
    //screenStack.push_back(new GuiScreen("gui/main"));
}

GuiTilemap::~GuiTilemap()
{
    while(!screenStack.empty()) {
        
        delete screenStack.back();
        screenStack.erase(--screenStack.end());
    }
}

void GuiTilemap::draw()
{
    ASSERT(globalConfInstance->getRenderingMode() == Conf::SOFTWARE);
    BITMAP* to = softwareRenderer.screen;
    
    GuiScreen *guiScreen = 0;
    
    if(!screenStack.empty())
        guiScreen = screenStack.back();

    tiles.draw();
    
    // Draw build menu bg
    ASSERT(handles.get_bmp());
    BITMAP* hnd = handles.get_bmp()->getBitmap();
    for(int y = 0; y < brow_h; y += 32)
    {
        bg.get_bmp()->blitToScreen(settings.screenWidth - 64, y);
        bg.get_bmp()->blitToScreen(settings.screenWidth - 32, y);
    }
    masked_blit(hnd, to, 0, 0, settings.screenWidth - 64, 0, 25, 25);
    masked_blit(hnd, to, 0, 25, settings.screenWidth - 64, brow_h - 25, 25, 25);
    
    // Draw build menu buttons
    for(int i = 0; i < 4; i++)
    {
        tiles.buildings.get_bmp(i + 1)->blitToScreen(settings.screenWidth - tiles.buildings.get_bmp(i + 1)->getw(), i * 32 + 15);
    }
    ASSERT(tiles.train.get_bmp());
    tiles.train.get_bmp(1)->blitToScreen(settings.screenWidth - 64, 4 * 32 + 15);
    tiles.train.get_bmp(0)->blitToScreen(settings.screenWidth - 32, 4 * 32 + 15);
    ASSERT(tiles.rails.get_bmp());
    masked_blit(tiles.rails.get_bmp()->getBitmap(), to, 32 * 3, 0, settings.screenWidth - 32, 5 * 32 + 15, 32, 32);
    
    //Print(settings.screenWidth - 10, settings.screenHeight - 1, Tint(), Print::RIGHT) << "Money: " << world.users.begin()->second.items[7];
    
    if(settings.fpsDisplay)
        Print(10, settings.screenHeight - 10) << "Fps: " << currentFPS;
    
    if(guiScreen)
        guiScreen->draw();
    
    globalConfInstance->flip();
}

GuiTilemap::Options GuiTilemap::think(Gui &caller)
{
    GuiScreen *screen = 0;
    
    if(!screenStack.empty())
        screen = screenStack.back();
    
    if(screen)
        handleActions();
        
    int status = 0;
    if(!host.poll(&status))
    {
        message(net::getError());
        return END;
    }
    status = host.check();
    if(status != 0)
    {
        message(errors::list[status]);
    }
        
    if(key[KEY_LEFT] || mouse_x < 10)
        tiles.cam_x -= 2;
    if(key[KEY_RIGHT] || (mouse_x > settings.screenWidth - 10 && mouse_y > brow_h) || mouse_x > settings.screenWidth - 3)
        tiles.cam_x += 2;
    if(key[KEY_UP] || mouse_y < 10)
        tiles.cam_y -= 2;
    if(key[KEY_DOWN] || mouse_y > settings.screenHeight - 10)
        tiles.cam_y += 2;
        
    /// Top left corner of the screen in world co-ords
    int s_t = tiles.cam_y - settings.screenWidth / 2, s_l = tiles.cam_x - settings.screenHeight / 2;
    tiles.caret_x = (mouse_x + s_l) / 32;
    tiles.caret_y = (mouse_y + s_t) / 32;
    
    if(mouse_b & 1 && !(old_mouse_b & 1))
    {
        if(mouse_x > settings.screenWidth - 64 && mouse_y < brow_h)
        {
            int item = (mouse_y - 15) / 32;
            switch(item)
            {
                case 0: cursor_state = 1; building = 1; tiles.caret_w = 1; tiles.caret_h = 1; break;
                case 1: cursor_state = 1; building = 2; tiles.caret_w = 2; tiles.caret_h = 1; break;
                case 2: cursor_state = 1; building = 3; tiles.caret_w = 1; tiles.caret_h = 1; break;
                case 3: cursor_state = 1; building = 4; tiles.caret_w = 2; tiles.caret_h = 1; break;
                case 4: cursor_state = 2; tiles.caret_w = 1; tiles.caret_h = 1; break;
                case 5: cursor_state = 3; tiles.caret_w = 1; tiles.caret_h = 1; break;
            }
        }
        else
        {
            if(cursor_state == 0)
            {
                int world_x = mouse_x + s_l, world_y = mouse_y + s_t;
                int wt_x = world_x / 32, wt_y = world_y / 32;
                if(key[KEY_LCONTROL])
                {
                    bool foundsomething = false;
                    for(map<unsigned int, Building>::iterator i = world.buildings.begin(); i != world.buildings.end(); i++)
                    {
                        //!! This is quite a magic number. Assumes 2-wide buildings are evenly-numbered
                        if(i->second.y == wt_y && (i->second.x == wt_x || 
                            (i->second.type % 2 == 0 && i->second.x == wt_x - 1)))
                        {
                            host.removeBuilding(i->first);
                            foundsomething = true;
                            break;
                        }
                    }
                    if(!foundsomething)
                    {
                        for(map<unsigned int, Track>::iterator i = world.tracks.begin(); i != world.tracks.end(); i++)
                        {
                            //!! This is quite a magic number. Assumes 2-wide buildings are evenly-numbered
                            if(i->second.y == wt_y && i->second.x == wt_x)
                            {
                                host.removeTrack(i->first);
                                foundsomething = true;
                                break;
                            }
                        }
                    }
                }
                else
                {
                    for(map<unsigned int, City>::iterator i = world.cities.begin(); i != world.cities.end(); i++)
                    {
                        if(i->second.x <= wt_x && wt_x <= i->second.x + 3 &&
                            i->second.y <= wt_y && wt_y <= i->second.y + 2)
                        {
                            out << "click " << i->first << "\n";
                        }
                    }
                }
            }
            else if(cursor_state == 1)
                host.buyBuilding(building, tiles.caret_x, tiles.caret_y);
            else if(cursor_state == 2)
            {
                Track t;
                out << world.users.size() << "\n";
                t.player = 1;
                t.x = tiles.caret_x;
                t.y = tiles.caret_y;
                out << "add track: " << world.addTrack(t) << "\n";
//                 host.buyTrain(tiles.caret_x, tiles.caret_y);
            }
            else if(cursor_state == 3)
                host.buyTrack(tiles.caret_x, tiles.caret_y);
            if(!key[KEY_LSHIFT])
            {
                cursor_state = 0;
                tiles.caret_w = 0;
                tiles.caret_h = 0;
            }
        }
    }
    if(mouse_b & 2)
    {
        cursor_state = 0;
        tiles.caret_w = 0;
        tiles.caret_h = 0;
    }
    old_mouse_b = mouse_b;
    
    if(key[KEY_ESC])
        return END;
    
    return CONTINUE;
}

void GuiTilemap::handleActions()
{
    static Settings settings;
    
    if(screenStack.empty())
        return;
    
    GuiScreen *screen = screenStack.back();
    
    screen->poll();
    
    if(screen->actionWaiting()) {
        
        Widget *commander;
        
        string action = screen->getAction(&commander);
        
        PerformAction: {
            
            out << "Action \"" << action << "\" activated.\n";
            
            if(commander->attributes["special"] == "disabled") {
                
                out << "Action canceled (the widget was disabled).\n";
            }
            else if(commander->attributes.count("requireAttribute") && commander->attributes[commander->attributes["requireAttribute"]].empty()) {
                
                out << "Action canceled (a requried attribute was missing from widget).\n";
            }
            else if(action == "runLogic") {
                
                Widget *widget = 0;
                
                widget = screen->findByKey("name", commander->attributes["logic"]);
                
                if(widget) {
                    
                    action = widget->attributes["activationAction"];
                    commander = widget;
                    
                    goto PerformAction;
                }
                else
                    out << "Warning, unable to find Logic widget for runLogic action.\n";
            }
            else if(action == "play") {
                
                // TODO
            }
            else if(action == "editor") {
                
                // TODO
            }
            else if(action == "load") {
                
                // TODO
            }
            else if(action == "previous") {
                
                int prevcount = atoi(commander->attributes["prevcount"].c_str());
                
                if(prevcount < 0) {
                    
                    while(screenStack.size() > (unsigned)abs(prevcount) + 1)
                        screenStack.erase(----screenStack.end());
                }
                else {
                        
                    for(int i = prevcount - 1; i > 0; i--) {
                        
                        if(screenStack.size() <= 1)
                            break;
                        
                        screenStack.erase(----screenStack.end());
                    }
                }
                
                if(screenStack.size() <= 1)
                    ;//TODO (quit)
                
                //TODO
            }
            else if(action == "quit") {
                
                // TODO
            }
            else if(action == "toggleMute") {
                
                if(commander->attributes["checked"] == "0")
                    settings.mute = false;
                else
                    settings.mute = true;
            }
            else if(action == "toggleParticles") {
                
                if(commander->attributes["checked"] == "0")
                    settings.particleEffects = false;
                else
                    settings.particleEffects = true;
            }
            else if(action == "toggleFullscreen") {
                
                if(commander->attributes["checked"] == "0")
                    settings.fullscreen = false;
                else
                    settings.fullscreen = true;
            }
            else {
                
                out << "Warning: \"" << action << "\" could not be found.\n";
            }
        }
    }
}
