#include "guiwindow.h"
#include "print.h"
#include "log.h"
using namespace std;

GuiWindow::GuiWindow(float x_, float y_, int w_, int h_)
    : x(x_), y(y_), w(w_), h(h_), background("gui/background/"),
      hwall("gui/borders/hwall/"), vwall("gui/borders/vwall/"),
      corner("gui/borders/corner/"), ltitle("gui/titlebar/left/"),
      ctitle("gui/titlebar/center/"), rtitle("gui/titlebar/right/")
{

}

void GuiWindow::drawBackground(int scroll, const Tint &tint)
{
    Bitmap *bmp = background.get_bmp();
    
    scroll %= 100;
    
    if(bmp)
        bmp->tileBlitToScreen(0, 0, 640, 480, 0, scroll / 100.0, tint);
}

void GuiWindow::drawForeground(const string &title, bool withGradient, const Tint &tint)
{
    int x = (int)this->x;
    int y = (int)this->y;
    
    if(withGradient)
        Bitmap::gradient(x, y, w, h, Tint(33 / 255.0, 33 / 255.0, 33 / 255.0, tint.a - .5), Tint(115 / 255.0, 115 / 255.0, 115 / 255.0, tint.a - .5), Tint(190 / 255.0, 190 / 255.0, 190 / 255.0, tint.a - .5), Tint(115 / 255.0, 115 / 255.0, 115 / 255.0, tint.a - .5));
    
    Bitmap *hwall = this->hwall.get_bmp();
    Bitmap *vwall = this->vwall.get_bmp();
    Bitmap *corner = this->corner.get_bmp();
    
    if(hwall) {
        
        hwall->tileBlitToScreen(x, y, w, 16, 0, 0, tint);
        hwall->tileBlitToScreen(x, y + h - 16, w, 16, 0, 0, tint);
    }
    
    if(vwall) {
        
        vwall->tileBlitToScreen(x, y, 16, h, 0, 0, tint);
        vwall->tileBlitToScreen(x + w - 16, y, 16, h, 0, 0, tint);
    }
    
    if(corner) {
        
        corner->specialBlitToScreen(x - 44, y - 44, 1, 1, 0, tint);
        corner->specialBlitToScreen(x + w - 20, y - 44, 1, 1, .25, tint);
        corner->specialBlitToScreen(x + w - 20, y + h - 20, 1, 1, .5, tint);
        corner->specialBlitToScreen(x - 44, y + h - 20, 1, 1, .75, tint);
    }
    
    if(!title.empty()) {
        
		int tmp;
        FontRenderer &rend = Print::getFontRenderer();
        
        int sw = rend.stringWidth(title);
        
        if(ltitle.get_bmp())
            ltitle.get_bmp()->blitToScreen(x + w / 2 - sw / 2 - 32, y - 35, tint);
        
		tmp=x+w/2-sw/2;
		
        if(ctitle.get_bmp())
            ctitle.get_bmp()->specialBlitToScreen(tmp, y - 36, sw / 32.0, 1, 1, tint);
        
		if(rtitle.get_bmp())
            rtitle.get_bmp()->blitToScreen(tmp+sw, y - 35, tint);
        
        rend.drawString(title, tmp, y - 18, tint);
    }
}
