#include "input.h"
#include <map>
#include <deque>
using namespace std;

namespace {
    
    typedef deque<Input*> InstanceStack;
    typedef map<char, InstanceStack> CharList;
    typedef map<int, InstanceStack> KeycodeList;
    
    typedef InstanceStack::iterator InstanceItr;
    typedef CharList::iterator CharItr;
    typedef KeycodeList::iterator KeycodeItr;
    
    CharList charList;
    KeycodeList keycodeList;
    
    void removeInstance(InstanceStack &instanceStack, Input *instance)
    {
        for(InstanceItr itr = instanceStack.begin(); itr != instanceStack.end(); ) {
            
            if(*itr == instance) {
                
                itr = instanceStack.erase(itr);
            }
            else
                ++itr;
        }
    }
}

void Input::requestCharacter(char character)
{
    charList[character].push_front(this);
}

void Input::requestCharacters(const string &characters)
{
    for(string::const_iterator itr = characters.begin(); itr != characters.end(); ++itr) {
        
        charList[*itr].push_front(this);
    }
}

void Input::unrequestCharacter(char character)
{
    CharItr loc = charList.find(character);
    
    if(loc == charList.end())
        return;
    
    InstanceStack &instanceStack = loc->second;
    
    for(InstanceItr itr = instanceStack.begin(); itr != instanceStack.end(); ++itr) {
        
        if(*itr == this) {
            
            instanceStack.erase(itr);
            break;
        }
    }
}

void Input::unrequestCharacters(const string &characters)
{
    for(string::const_iterator itr = characters.begin(); itr != characters.end(); ++itr) {
        
        unrequestCharacter(*itr);
    }
}

bool Input::characterWaiting()
{
    getInput();
    
    return !characters.empty();
}

char Input::getCharacter()
{
    if(characters.empty())
        return '\0';
    
    char ret = characters.front();
    
    characters.pop();
    
    return ret;
}

void Input::requestKeycode(int keycode)
{
    keycodeList[keycode].push_front(this);
}

void Input::unrequestKeycode(int keycode)
{
    KeycodeItr loc = keycodeList.find(keycode);
    
    if(loc == keycodeList.end())
        return;
    
    InstanceStack &instanceStack = loc->second;
    
    for(InstanceItr itr = instanceStack.begin(); itr != instanceStack.end(); ++itr) {
        
        if(*itr == this) {
            
            instanceStack.erase(itr);
            break;
        }
    }
}

bool Input::keycodeWaiting()
{
    getInput();
    
    return !keycodes.empty();
}

int Input::getKeycode()
{
    if(keycodes.empty())
        return 0;
    
    int ret = keycodes.front();
    
    keycodes.pop();
    
    return ret;
}

Input::~Input()
{
    // Remove this instance from all possible locations on the keycode
    // and character input stacks.
    
    for(CharItr itr = charList.begin(); itr != charList.end(); ++itr)
        removeInstance(itr->second, this);
    
    for(KeycodeItr itr = keycodeList.begin(); itr != keycodeList.end(); ++itr)
        removeInstance(itr->second, this);
}

void Input::getInput()
{
    while(keypressed()) {
        
        int val = readkey();
        int keycode = val >> 8;
        char character = val & 0xff;
        
        if(keycode) {
            
            KeycodeItr loc = keycodeList.find(keycode);
            
            if(loc != keycodeList.end() && !loc->second.empty())
                loc->second.front()->keycodes.push(keycode);
        }
        
        if(character) {
            
            CharItr loc = charList.find(character);
            
            if(loc != charList.end() && !loc->second.empty())
                loc->second.front()->characters.push(character);
        }
    }
}
