#include "mutex.h"
#include <errno.h>
using namespace std;

Mutex::Mutex()
{
#ifdef WIN32
    mutex = CreateMutex( 
        NULL,                       // default security attributes
        FALSE,                      // initially not owned
        NULL);                      // unnamed mutex
    
    if(mutex == NULL)
        throw "CreateMutex";
#else
    if(0 != pthread_mutex_init(&pthread_mutex, 0))
        throw "pthread_mutex_init";
#endif
}

Mutex::~Mutex()
{
#ifdef WIN32
    CloseHandle(mutex);
#else
    if(0 != pthread_mutex_destroy(&pthread_mutex))
        throw "pthread_mutex_destroy";
#endif
}

void Mutex::lock()
{
#ifdef WIN32
    DWORD res = WaitForSingleObject(mutex, INFINITE);
    
    if(res != WAIT_OBJECT_0)
        throw "WaitForSingleObject: lock";
#else
    if(0 != pthread_mutex_lock(&pthread_mutex))
        throw "pthread_mutex_lock";
#endif
}

bool Mutex::tryToLock()
{
#ifdef WIN32
    DWORD res = WaitForSingleObject(mutex, 0);
    
    if(res == WAIT_OBJECT_0)
        return true;
    
    if(res == WAIT_TIMEOUT)
        return false;
    
    throw "WaitForSingleObject: tryToLock";
#else
    switch(pthread_mutex_trylock(&pthread_mutex)) {
        
        case 0: return true;
        default: throw "pthread_mutex_trylock";
        case EBUSY: break;
    }
#endif
    
    return false;
}

void Mutex::unlock()
{
#ifdef WIN32
    if(!ReleaseMutex(mutex))
        throw "ReleaseMutex";
#else
    if(0 != pthread_mutex_unlock(&pthread_mutex))
        throw "pthread_mutex_unlock";
#endif
}
