#include "particle.h"
#include "log.h"
#include <map>
#include <cmath>
using namespace std;

#define PARTICLE_CL_FADE_IN   (1.0-(cur/ini))
#define PARTICLE_CL_FADE_OUT  (cur/ini)
#define PARTICLE_CL_BLINK     ((ttl[0]<(ttl[1]/2.0f))?(cur/(ini/2.0f)):((ini/2.0f)-((cur-(ini/2.0))))/(ini/2.0f))

namespace {
    
    map< pair<string, datafile_t*>, animation* > graphics;
}

void Particle::load(const string &objname, datafile_t *datafile)
{
    pair<string, datafile_t*> p(objname, datafile);
    map< pair<string, datafile_t*>, animation* >::iterator itr = graphics.find(p);
    
    if(itr != graphics.end())
        graphic = itr->second;
    else
        graphics.insert(make_pair(p, graphic = new animation(objname, datafile)));
}

void Particle::cleanup()
{
    map< pair<string, datafile_t*>, animation* >::iterator itr = graphics.begin();
    
    for( ; itr != graphics.end(); ++itr)
        delete itr->second;
    
    graphics.clear();
}

void Particle::draw(const Tint &screenTint)
{
    Tint tint(screenTint.r, screenTint.g, screenTint.b, alpha * screenTint.a, logic & 0x40);
    
    if(graphic->get_bmp())
        graphic->get_bmp()->specialBlitToScreen((int)x - (int)width / 2, (int)y - (int)height / 2, width / graphic->get_bmp()->getw(), height / graphic->get_bmp()->geth(), dir / (2 * M_PI), tint);
}

void Particle::update()
{
    float cur, ini;
    
    cur = ttl[0];
    ini = ttl[1];
    
    switch(logic & 0x3f) {
        default:						
            break;
        case 1:
            vel=sqrt(((x-xStart)*(x-xStart))+((y-yStart)*(y-yStart)))/5.0;
            height++;
            width++;
            x+=cos(dir)*vel;
            y+=sin(dir)*vel;
            alpha=PARTICLE_CL_FADE_IN;
            logic |= 0x40;
            break;
        case 2:
            height+=(80-height)/16;
            width+=(80-width)/16;
            alpha=PARTICLE_CL_FADE_OUT;
            logic &= ~0x40;
            break;				
        case 3:
            x+=cos(dir)*vel;
            y+=sin(dir)*vel;
            alpha=PARTICLE_CL_FADE_OUT;
            logic &= ~0x40;
            break;
        case 4:
            x+=cos(dir)*vel;
            y+=sin(dir)*vel;
            alpha=PARTICLE_CL_FADE_IN;
            logic |= 0x40;
            break;
        case 6:
            vel=sqrt(((x-xStart)*(x-xStart))+((y-yStart)*(y-yStart)))/5.0;
            height++;
            width++;
            x+=cos(dir)*vel;
            y+=sin(dir)*vel;
            alpha=PARTICLE_CL_BLINK;
            logic &= ~0x40;
            break;
        case 7:
            dir+=.03;
            alpha=PARTICLE_CL_BLINK;
            logic &= ~0x40;
            break;
        case 8:
            x+=cos(dir)*vel;
            y+=sin(dir)*vel;
            alpha=PARTICLE_CL_FADE_OUT;
            logic &= ~0x40;
            break;
        case 9:
            alpha=PARTICLE_CL_BLINK;
            logic |= 0x40;
            break;
        case 10:
            dir+=.03;
            alpha=PARTICLE_CL_BLINK;
            alpha/=2.0;
            logic |= 0x40;
            break;
        case 11:
            dir-=.03;
            alpha=PARTICLE_CL_BLINK;
            alpha/=2.0;
            logic |= 0x40;
            break;
        case 12:
            width++;
            height++;
            alpha=PARTICLE_CL_FADE_OUT;
            logic |= 0x40;
            break;
        case 13:
            dir+=.03;
            alpha=PARTICLE_CL_FADE_OUT;
            logic |= 0x40;
            break;
        case 14:
            dir-=.03;
            alpha=PARTICLE_CL_FADE_OUT;
            logic |= 0x40;
            break;		
        case 15:
            width*=0.9;
            height*=0.9;
			if (width<4) width=4;
			if (height<4) height=4;
            alpha=PARTICLE_CL_FADE_OUT;
            logic |= 0x40;
            break;			
    }
}
