#include "sounds.h"
#include "log.h"
#include <map>
#include <sstream>
using namespace std;

namespace {
    
    map<string, SAMPLE*> sounds;
    
    float sfxVolume;
}

void Sounds::setSfxVolume(float volume)
{
    sfxVolume = volume;
}

float Sounds::getSfxVolume()
{
  return sfxVolume;
}

void Sounds::load(const string &objdir, datafile_t *datafile)
{
    DATAFILE *datf;
    SAMPLE *spl;
    
    for(int i = 0; 1; i++) {
        
        ostringstream os;
        
        os << objdir << i;
        
        spl = (SAMPLE*)datafile->loadfind(os.str(), &datf);
        
        if(!spl)
            break;
        
        sounds[get_datafile_property(datf, DAT_ID('S', 'O', 'N', 'D'))] = spl;
    }
}

void Sounds::unload()
{
    sounds.clear();
}

bool Sounds::play(const string &sound, const PlayInfo &playInfo)
{
    map<string, SAMPLE*>::iterator loc = sounds.find(sound);
    
    if(loc != sounds.end() && loc->second)
        return -1 != play_sample(loc->second, (int)(sfxVolume * 0xff), (int)(playInfo.pan * 0xff), (int)(playInfo.frequency * 1000), 0);
    else
        return false;
}

bool Sounds::loop(const string &sound, const PlayInfo &playInfo)
{
    map<string, SAMPLE*>::iterator loc = sounds.find(sound);
    
    if(loc != sounds.end() && loc->second)
        return -1 != play_sample(loc->second, (int)(sfxVolume * 0xff), (int)(playInfo.pan * 0xff), (int)(playInfo.frequency * 1000), 1);
    else
        return false;
}

bool Sounds::adjust(const string &sound, bool looped, const PlayInfo &playInfo)
{
    map<string, SAMPLE*>::iterator loc = sounds.find(sound);
    
    if(loc != sounds.end() && loc->second)
        adjust_sample(loc->second, (int)(sfxVolume * 0xff), (int)(playInfo.pan * 0xff), (int)(playInfo.frequency * 1000), looped);
    else
        return false;
    
    return true;
}

void Sounds::stop(const string &sound)
{
    map<string, SAMPLE*>::iterator loc = sounds.find(sound);
    
    if(loc != sounds.end() && loc->second)
        stop_sample(loc->second);
}

void Sounds::stopAll()
{
    for(map<string, SAMPLE*>::iterator itr = sounds.begin(); itr != sounds.end(); ++itr) {
        
        if(itr->second)
            stop_sample(itr->second);
    }
}

