#include "stopwatch.h"
#include "timer.h"
#include <list>
#include <algorithm>
using namespace std;

namespace {
    
    list<stopwatch*> instances;
    unsigned int pauseTime = 0;
}

void stopwatch::pauseAll()
{
    for(list<stopwatch*>::iterator itr = instances.begin(); itr != instances.end(); ++itr)
        (*itr)->pause();
}

void stopwatch::unpauseAll()
{
    for(list<stopwatch*>::iterator itr = instances.begin(); itr != instances.end(); ++itr)
        (*itr)->unpause();
}

stopwatch::stopwatch()
    : start_time(0), pause_time(0), ticking(0), isPaused(0)
{
    instances.push_back(this);
}

stopwatch::~stopwatch()
{
    instances.erase(find(instances.begin(), instances.end(), this));
}

void stopwatch::start(unsigned int time)
{
    start_time = timer.msecs();
    ticking = true;
    pause_time = time;
}

void stopwatch::pause()
{
    if(!isPaused) {
        
        isPaused = true;
        timeOfPausing = timer.msecs();
    }
}

void stopwatch::unpause()
{
    if(isPaused) {
        
        isPaused = false;
        start_time += timer.msecs() - timeOfPausing;
    }
}

void stopwatch::reset()
{
    ticking = false;
}

bool stopwatch::check()
{
    if(isPaused)
        if(!ticking || start_time + pause_time <= timeOfPausing)
            return true;
        else
            return false;
    
    if(!ticking || start_time + pause_time <= timer.msecs()) {
        
        if(!pauseTime)
            ticking = false;
        
        return true;
    }
    
    return false;
}

unsigned int stopwatch::msecs()
{
    Timer::time_t t;
    
    if(isPaused)
        t = timeOfPausing;
    else
        t = timer.msecs();
    
    if(ticking && t > start_time)
        return t - start_time;
    
    return pause_time;
}
