#include "terminal.h"
#include "world.h"
#include <set>
using namespace std;


Terminal::Terminal()
    : owner_id(0), city(0)
{
}

Terminal::Terminal(unsigned int owner_id_, unsigned int city_, unsigned int track_)
    : owner_id(owner_id_), city(city_), track(track_)
{
}

Terminal::~Terminal()
{
}

void Terminal::list_exits(std::vector<CityLink>& exits)
{
    exits.insert(exits.end(), tracks.begin(), tracks.end());
}

unsigned int Terminal::get_cost(unsigned int city)
{
    for(vector<CityLink>::iterator i = tracks.begin(); i != tracks.end(); i++)
    {
        if(i->cityA == city || i->cityB == city)
            return i->cost;
    }
    return 0;
}

void Terminal::adjust_ticket(unsigned int destcity, unsigned int newcost, bool updateboth)
{
    for(vector<CityLink>::iterator i = tracks.begin(); i != tracks.end(); i++)
    {
        if(i->cityA == destcity || i->cityB == destcity)
        {
            i->cost = newcost;
            if(updateboth)
            {
                throw "Terminal::adjust_ticket: update_both is a royal mess";
            }
            return;
        }
    }
}

void Terminal::recalculate_tracks()
{
    if(!track)
        return;
        
    set<unsigned int> found;
    vector<unsigned int> buildings;
        
    {
        vector<Track*> todo;
        
        todo.push_back(&world.tracks[track]);
        
        while(todo.size())
        {
            Track* t = todo.back();
            todo.pop_back();
            
            if(t->buildings[0])
                buildings.push_back(t->buildings[0]);
            if(t->buildings[1])
                buildings.push_back(t->buildings[1]);
            
            vector<unsigned int> adjacent;
            t->find_adjacent(adjacent);
            
            for(vector<unsigned int>::iterator i = adjacent.begin(); i != adjacent.end(); i++)
            {
                if(found.find(*i) == found.end())
                {
                    todo.push_back(&world.tracks[*i]);
                    found.insert(*i);
                }
            }
        }
    }
    
    vector<Terminal*> t_list;
    for(map<unsigned int, Terminal>::iterator i = world.terminals.begin(); i != world.terminals.end(); i++)
    {
        if(found.find(i->second.track) != found.end())
        {
            t_list.push_back(&i->second);
        }
    }
    
    for(vector<Terminal*>::iterator i = t_list.begin(); i != t_list.end(); i++)
    {
        (*i)->propagate(t_list, buildings);
    }
}

void Terminal::propagate(std::vector<Terminal*>& newtracks, std::vector<unsigned int>& buildings)
{
    this->buildings.clear();
    this->buildings.insert(this->buildings.end(), buildings.begin(), buildings.end());
    
    vector<CityLink> tracks;
    for(vector<Terminal*>::iterator i = newtracks.begin(); i != newtracks.end(); i++)
    {
        if(*i == this)
            continue;
        CityLink c;
        c.player = owner_id;
        c.cityA = city;
        c.cityB = (*i)->city;
        c.cost = 100;
        for(vector<CityLink>::iterator j = this->tracks.begin(); j != this->tracks.end(); j++)
        {
            if((j->cityA == city && j->cityB == (*i)->city) ||
                (j->cityB == city && j->cityA == (*i)->city))
            {
                c.cost = j->cost;
                this->tracks.erase(j);
                break;
            }
        }
        tracks.push_back(c);
    }
    this->tracks.clear();
    this->tracks.insert(this->tracks.end(), tracks.begin(), tracks.end());
}
