#include "tilemap.h"
#include "settings.h"
#include "log.h"
#include "print.h"
#include "conf.h"
#include "software.h"
#include "world.h"
using namespace std;

Tilemap::Tilemap()
    : cam_x(0), cam_y(0), caret_x(0), caret_y(0), caret_w(0), caret_h(0), caret_color(-1),
    grass("tiles/ground/"), rails("tiles/rails/"), town("tiles/town/"), resources("tiles/resources/"), buildings("tiles/buildings/"), train("tiles/train/")
{
}

Tilemap::~Tilemap()
{
}

void Tilemap::draw()
{
    ASSERT(globalConfInstance->getRenderingMode() == Conf::SOFTWARE);
    BITMAP* to = softwareRenderer.screen;
    
    /// Top left corner of the screen in world co-ords
    int s_t = cam_y - settings.screenWidth / 2, s_l = cam_x - settings.screenHeight / 2;
    if(s_t < 0)
    {
        cam_y -= s_t;
        s_t = 0;
    }
    if(s_l < 0)
    {
        cam_x -= s_l;
        s_l = 0;
    }
    
    if(grass.get_bmp())
    {
        Bitmap* gr = grass.get_bmp();
        int sx = -(s_l % gr->getw());
        int odd_off = gr->getw() / 2;
        bool odd_row = (s_t / gr->geth()) % 2;
        for(int y = -(s_t % gr->geth()); y < settings.screenHeight; y += gr->geth(), odd_row = !odd_row)
        {
            for(int x = sx - (odd_row ? odd_off : 0); x < settings.screenWidth; x += gr->getw())
            {
                gr->blitToScreen(x, y, Tint(), true);
            }
        }
    }
    
    ASSERT(rails.get_bmp());
    BITMAP* rls = rails.get_bmp()->getBitmap();
    ASSERT(rls);
    for(map<unsigned int, Track>::iterator i = world.tracks.begin(); i != world.tracks.end(); i++)
    {
        Track& t = i->second;
        int off = 9;
        switch(t.directions)
        {
            case 1: off = 8; break;
            case 2: off = 9; break;
            case 3: off = 3; break;
            case 4: off = 6; break;
            case 5: off = 5; break;
            case 6: off = 2; break;
            case 8: off = 7; break;
            case 9: off = 4; break;
            case 10: off = 0; break;
            case 12: off = 1; break;
        }
        masked_blit(rls, to, off * 32, 0, t.x * 32 - s_l, t.y * 32 - s_t, 32, 32);
    }
    
    if(town.get_bmp())
    {
        for(map<unsigned int, City>::iterator i = world.cities.begin(); i != world.cities.end(); i++)
        {
            City& t = i->second;
            town.get_bmp()->blitToScreen(t.x * 32 - s_l, t.y * 32 - s_t, Tint(), true);
        }
    }
    
    if(resources.get_bmp(1))
    {
        for(map<unsigned int, Resource>::iterator i = world.resources.begin(); i != world.resources.end(); i++)
        {
            Resource& t = i->second;
            resources.get_bmp(t.type)->blitToScreen(t.x * 32 - s_l, t.y * 32 - s_t, Tint(), true);
        }
    }
    
    if(buildings.get_bmp(1))
    {
        for(map<unsigned int, Building>::iterator i = world.buildings.begin(); i != world.buildings.end(); i++)
        {
            Building& t = i->second;
            buildings.get_bmp(t.type)->blitToScreen(t.x * 32 - s_l, t.y * 32 - s_t, Tint(), true);
        }
    }
    
    if(caret_w && caret_h)
    {
        rect(to, caret_x * 32 - s_l, caret_y * 32 - s_t, (caret_x + caret_w) * 32 - s_l, (caret_y + caret_h) * 32 - s_t, caret_color);
    }
}
