#include "timer.h"
#include "log.h"

#ifdef WIN32
#include <windows.h>
#else
#include <sys/time.h>
#endif

Timer timer;

Timer::Timer()
{
#ifdef WIN32
    startTime = GetTickCount();
#else
    gettimeofday(&startTime, 0);
#endif
}

Timer::time_t Timer::usecs()
{
#ifdef WIN32
    static bool onetime = 0;
    static unsigned long long freq = 0;
    
    if(!onetime) {
        
        onetime = true;
        QueryPerformanceFrequency((LARGE_INTEGER*)&freq);
    }
    
    QueryPerformanceCounter((LARGE_INTEGER*)&currentTime);
    
    currentTime *= 1000000;
    currentTime /= freq;
    
    return currentTime - startTime * 1000;
#else
    gettimeofday(&currentTime, 0);
    
    return (unsigned long)((currentTime.tv_sec - startTime.tv_sec) / 1000)
            + (unsigned long)((currentTime.tv_usec - startTime.tv_usec));
#endif
}

Timer::time_t Timer::msecs()
{
#ifdef WIN32
    currentTime = GetTickCount();
    
    return currentTime - startTime;
#else
    gettimeofday(&currentTime, 0);
    
    return (unsigned long)((currentTime.tv_sec - startTime.tv_sec) * 1000)
            + (unsigned long)((currentTime.tv_usec - startTime.tv_usec) / 1000);
#endif
}

Timer::time_t Timer::secs()
{
#ifdef WIN32
    currentTime = GetTickCount();
    
    return (currentTime - startTime) / 1000;
#else
    gettimeofday(&currentTime, 0);
    
    return (unsigned long)(currentTime.tv_sec - startTime.tv_sec);
#endif
}

